@@ -1,6 +1,6 @@
 ##############################################################################
 #
-# Copyright (c) 2005 Zope Corporation and Contributors.
+# Copyright (c) 2005 Zope Foundation and Contributors.
 # All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
@@ -13,20 +13,27 @@
 ##############################################################################
 """Mechanize-based Functional Doctest interfaces
 
-$Id: browser.py 85021 2008-03-30 23:49:20Z benji_york $
+$Id: browser.py 124525 2012-03-05 20:59:15Z menesis $
 """
+
 __docformat__ = "reStructuredText"
-from zope.testbrowser import interfaces
-import ClientForm
-from cStringIO import StringIO
-import mechanize
+
+
+import cStringIO
+import datetime
 import operator
 import re
 import sys
 import time
 import urllib2
 
-from zope import interface
+import ClientForm
+import mechanize
+import zope.interface
+
+import zope.testbrowser.cookies
+import zope.testbrowser.interfaces
+
 
 RegexType = type(re.compile(''))
 _compress_re = re.compile(r"\s+")
@@ -153,7 +160,7 @@ class PystoneTimer(object):
 
 class Browser(SetattrErrorsMixin):
     """A web user agent."""
-    interface.implements(interfaces.IBrowser)
+    zope.interface.implements(zope.testbrowser.interfaces.IBrowser)
 
     _contents = None
     _counter = 0
@@ -164,6 +171,7 @@ class Browser(SetattrErrorsMixin):
         self.mech_browser = mech_browser
         self.timer = PystoneTimer()
         self.raiseHttpErrors = True
+        self.cookies = zope.testbrowser.cookies.Cookies(self.mech_browser)
         self._enable_setattr_errors = True
 
         if url is not None:
@@ -190,6 +198,8 @@ class Browser(SetattrErrorsMixin):
         if self._contents is not None:
             return self._contents
         response = self.mech_browser.response()
+        if response is None:
+            return None
         old_location = response.tell()
         response.seek(0)
         self._contents = response.read()
@@ -208,16 +218,21 @@ class Browser(SetattrErrorsMixin):
 
         def get(self):
             headers = self.mech_browser.addheaders
-            return dict(headers).get(header_key, True)
+            value = dict(headers).get(header_key, True)
+            return {'False': False}.get(value, True)
 
         def set(self, value):
             headers = self.mech_browser.addheaders
             current_value = get(self)
             if current_value == value:
                 return
-            if header_key in dict(headers):
-                headers.remove((header_key, current_value))
-            headers.append((header_key, value))
+
+            # Remove the current header...
+            for key, header_value in headers[:]:
+                if key == header_key:
+                    headers.remove((key, header_value))
+            # ... Before adding the new one.
+            headers.append((header_key, {False: 'False'}.get(value, 'True')))
 
         return property(get, set)
 
@@ -286,6 +301,12 @@ class Browser(SetattrErrorsMixin):
 
     def addHeader(self, key, value):
         """See zope.testbrowser.interfaces.IBrowser"""
+        if (self.mech_browser.request is not None and
+            key.lower() in ('cookie', 'cookie2') and
+            self.cookies.header):
+            # to prevent unpleasant intermittent errors, only set cookies with
+            # the browser headers OR the cookies mapping.
+            raise ValueError('cookies are already set in `cookies` attribute')
         self.mech_browser.addheaders.append( (str(key), str(value)) )
 
     def getLink(self, text=None, url=None, id=None, index=0):
@@ -406,7 +427,7 @@ class Browser(SetattrErrorsMixin):
 
 
 class Link(SetattrErrorsMixin):
-    interface.implements(interfaces.ILink)
+    zope.interface.implements(zope.testbrowser.interfaces.ILink)
 
     def __init__(self, link, browser):
         self.mech_link = link
@@ -416,7 +437,7 @@ class Link(SetattrErrorsMixin):
 
     def click(self):
         if self._browser_counter != self.browser._counter:
-            raise interfaces.ExpiredError
+            raise zope.testbrowser.interfaces.ExpiredError
         self.browser._start_timer()
         self.browser.mech_browser.follow_link(self.mech_link)
         self.browser._stop_timer()
@@ -445,7 +466,7 @@ class Link(SetattrErrorsMixin):
 
 class Control(SetattrErrorsMixin):
     """A control of a form."""
-    interface.implements(interfaces.IControl)
+    zope.interface.implements(zope.testbrowser.interfaces.IControl)
 
     _enable_setattr_errors = False
 
@@ -496,7 +517,7 @@ class Control(SetattrErrorsMixin):
 
         def fset(self, value):
             if self._browser_counter != self.browser._counter:
-                raise interfaces.ExpiredError
+                raise zope.testbrowser.interfaces.ExpiredError
             if self.mech_control.type == 'file':
                 self.mech_control.add_file(value,
                                            content_type=self.content_type,
@@ -512,12 +533,12 @@ class Control(SetattrErrorsMixin):
             raise TypeError("Can't call add_file on %s controls"
                             % self.mech_control.type)
         if isinstance(file, str):
-            file = StringIO(file)
+            file = cStringIO.StringIO(file)
         self.mech_control.add_file(file, content_type, filename)
 
     def clear(self):
         if self._browser_counter != self.browser._counter:
-            raise interfaces.ExpiredError
+            raise zope.testbrowser.interfaces.ExpiredError
         self.mech_control.clear()
 
     def __repr__(self):
@@ -526,7 +547,7 @@ class Control(SetattrErrorsMixin):
 
 
 class ListControl(Control):
-    interface.implements(interfaces.IListControl)
+    zope.interface.implements(zope.testbrowser.interfaces.IListControl)
 
     @apply
     def displayValue():
@@ -540,7 +561,7 @@ class ListControl(Control):
 
         def fset(self, value):
             if self._browser_counter != self.browser._counter:
-                raise interfaces.ExpiredError
+                raise zope.testbrowser.interfaces.ExpiredError
             self.mech_control.set_value_by_label(value)
 
         return property(fget, fset)
@@ -576,7 +597,7 @@ class ListControl(Control):
     @property
     def controls(self):
         if self._browser_counter != self.browser._counter:
-            raise interfaces.ExpiredError
+            raise zope.testbrowser.interfaces.ExpiredError
         res = [controlFactory(i, self.mech_form, self.browser) for i in
                 self.mech_control.items]
         for s in res:
@@ -585,7 +606,7 @@ class ListControl(Control):
 
     def getControl(self, label=None, value=None, index=None):
         if self._browser_counter != self.browser._counter:
-            raise interfaces.ExpiredError
+            raise zope.testbrowser.interfaces.ExpiredError
 
         onlyOne([label, value], '"label" and "value"')
 
@@ -602,27 +623,27 @@ class ListControl(Control):
 
 
 class SubmitControl(Control):
-    interface.implements(interfaces.ISubmitControl)
+    zope.interface.implements(zope.testbrowser.interfaces.ISubmitControl)
 
     def click(self):
         if self._browser_counter != self.browser._counter:
-            raise interfaces.ExpiredError
+            raise zope.testbrowser.interfaces.ExpiredError
         self.browser._clickSubmit(self.mech_form, self.mech_control, (1,1))
         self.browser._changed()
 
 
 class ImageControl(Control):
-    interface.implements(interfaces.IImageSubmitControl)
+    zope.interface.implements(zope.testbrowser.interfaces.IImageSubmitControl)
 
     def click(self, coord=(1,1)):
         if self._browser_counter != self.browser._counter:
-            raise interfaces.ExpiredError
+            raise zope.testbrowser.interfaces.ExpiredError
         self.browser._clickSubmit(self.mech_form, self.mech_control, coord)
         self.browser._changed()
 
 
 class ItemControl(SetattrErrorsMixin):
-    interface.implements(interfaces.IItemControl)
+    zope.interface.implements(zope.testbrowser.interfaces.IItemControl)
 
     def __init__(self, item, form, browser):
         self.mech_item = item
@@ -634,7 +655,7 @@ class ItemControl(SetattrErrorsMixin):
     @property
     def control(self):
         if self._browser_counter != self.browser._counter:
-            raise interfaces.ExpiredError
+            raise zope.testbrowser.interfaces.ExpiredError
         res = controlFactory(
             self.mech_item._control, self.mech_form, self.browser)
         self.__dict__['control'] = res
@@ -653,7 +674,7 @@ class ItemControl(SetattrErrorsMixin):
 
         def fset(self, value):
             if self._browser_counter != self.browser._counter:
-                raise interfaces.ExpiredError
+                raise zope.testbrowser.interfaces.ExpiredError
             self.mech_item.selected = value
 
         return property(fget, fset)
@@ -664,7 +685,7 @@ class ItemControl(SetattrErrorsMixin):
 
     def click(self):
         if self._browser_counter != self.browser._counter:
-            raise interfaces.ExpiredError
+            raise zope.testbrowser.interfaces.ExpiredError
         self.mech_item.selected = not self.mech_item.selected
 
     def __repr__(self):
@@ -675,7 +696,7 @@ class ItemControl(SetattrErrorsMixin):
 
 class Form(SetattrErrorsMixin):
     """HTML Form"""
-    interface.implements(interfaces.IForm)
+    zope.interface.implements(zope.testbrowser.interfaces.IForm)
 
     def __init__(self, browser, form):
         """Initialize the Form
@@ -712,7 +733,7 @@ class Form(SetattrErrorsMixin):
     def submit(self, label=None, name=None, index=None, coord=(1,1)):
         """See zope.testbrowser.interfaces.IForm"""
         if self._browser_counter != self.browser._counter:
-            raise interfaces.ExpiredError
+            raise zope.testbrowser.interfaces.ExpiredError
         form = self.mech_form
         if label is not None or name is not None:
             intermediate, msg = self.browser._get_all_controls(
@@ -735,7 +756,7 @@ class Form(SetattrErrorsMixin):
     def getControl(self, label=None, name=None, index=None):
         """See zope.testbrowser.interfaces.IBrowser"""
         if self._browser_counter != self.browser._counter:
-            raise interfaces.ExpiredError
+            raise zope.testbrowser.interfaces.ExpiredError
         intermediate, msg = self.browser._get_all_controls(
             label, name, (self.mech_form,), include_subcontrols=True)
         control, form = disambiguate(intermediate, msg, index)
